"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebpackPluginStartElectron = void 0;
const url_1 = require("url");
const webpack_1 = require("webpack");
const path_1 = __importDefault(require("path"));
const child_process_1 = require("child_process");
const events_1 = require("events");
const electron_1 = __importDefault(require("electron"));
const readline_1 = __importDefault(require("readline"));
class WebpackPluginStartElectron {
    constructor(opts = {}) {
        this.electronProcess = null;
        this.appPath = null;
        this.mainReady = false;
        this.rendererReady = false;
        if (WebpackPluginStartElectron.instance) {
            return WebpackPluginStartElectron.instance;
        }
        this.opts = {
            fileUrl: false,
            electronLiveReload: true,
            ...opts,
        };
        this.setupKeypressListeners();
        WebpackPluginStartElectron.instance = this;
    }
    apply(compiler) {
        this.logger = compiler.getInfrastructureLogger('webpack-plugin-start-electron');
        if (compiler.options.target === 'electron-renderer') {
            this.rendererCompiler = compiler;
        }
        if (compiler.options.target === 'electron-main') {
            this.mainCompiler = compiler;
        }
        if (this.rendererCompiler && this.mainCompiler) {
            const mainEntry = Object.keys(this.mainCompiler.options.entry)[0];
            const rendererEntries = Object.keys(this.rendererCompiler.options.entry);
            const rendererOutputPath = this.rendererCompiler.options.output
                .path;
            const devServerPort = this.rendererCompiler.options.devServer?.port ?? 4242;
            const env = Object.fromEntries(rendererEntries.map((name) => {
                return [
                    `COMPASS_${name.toUpperCase()}_RENDERER_URL`,
                    this.opts.fileUrl
                        ? (0, url_1.pathToFileURL)(path_1.default.join(rendererOutputPath, `${name}.html`))
                        : `http://localhost:${devServerPort}/${name}.html`,
                ];
            }));
            const envPlugin = new webpack_1.EnvironmentPlugin(env);
            envPlugin.apply(this.mainCompiler);
            this.mainCompiler.hooks.afterEmit.tapPromise('WebpackPluginStartElectron', async (compilation) => {
                const entryChunk = compilation.namedChunks.get(mainEntry);
                const entryChunkFilename = entryChunk?.files.values().next().value;
                if (!compilation.options.output.path) {
                    throw new Error('Output path is required');
                }
                this.appPath = path_1.default.join(compilation.options.output.path, entryChunkFilename);
                if (this.opts.electronLiveReload) {
                    await this.stopElectron();
                    this.startElectron();
                }
            });
            this.mainCompiler.hooks.afterDone.tap('WebpackPluginStartElectron', () => {
                this.mainReady = true;
                this.startElectron();
            });
            this.rendererCompiler.hooks.afterDone.tap('WebpackPluginStartElectron', () => {
                this.rendererReady = true;
                this.startElectron();
            });
        }
    }
    async stopElectron() {
        if (!this.electronProcess) {
            return;
        }
        this.logger.info('Stopping currently running app');
        await this.kill(this.electronProcess);
        this.electronProcess = null;
    }
    startElectron(extraArgs) {
        if (this.electronProcess) {
            return;
        }
        if (!this.mainReady || !this.rendererReady) {
            return;
        }
        if (!this.appPath) {
            throw new Error("Can't start the app: electron main path is not set");
        }
        extraArgs =
            extraArgs ??
                process.env.ELECTRON_EXTRA_ARGS?.split(' ').map((arg) => {
                    return arg.trim();
                });
        this.logger.info('Starting electron application');
        this.logger.info('- Ctrl+R to restart the main process');
        this.logger.info('- Ctrl+A to restart the main process with extra arguments');
        this.electronProcess = (0, child_process_1.spawn)(electron_1.default, [this.appPath, ...(extraArgs ?? [])], { stdio: 'inherit', env: process.env });
        let stderr = '';
        this.electronProcess.stderr
            ?.setEncoding('utf-8')
            .on('data', (chunk) => (stderr += chunk));
        this.electronProcess.on('exit', (code) => {
            this.electronProcess = null;
            if (code && code > 0) {
                this.logger.error('Electron app quit unexpectedly on start. Interrupting build.');
                throw new Error('Electron app quit unexpectedly:\n\n' + stderr);
            }
        });
    }
    async kill(childProcess, code = 'SIGTERM') {
        childProcess.kill(code);
        if (childProcess.exitCode === null && childProcess.signalCode === null) {
            await (0, events_1.once)(childProcess, 'exit');
        }
    }
    setupKeypressListeners() {
        if (!process.stdin.isTTY) {
            return;
        }
        process.stdin.setRawMode(true);
        readline_1.default.emitKeypressEvents(process.stdin);
        process.stdin.on('keypress', (str) => {
            if (str === '\x03') {
                this.electronProcess?.kill('SIGINT');
                process.emit('SIGINT');
                return;
            }
            if (str === '\x12') {
                void this.stopElectron().then(() => {
                    this.startElectron();
                });
                return;
            }
            if (str === '\x01') {
                const rl = readline_1.default.createInterface(process.stdin, process.stdout);
                rl.question('Enter extra args (e.g., --protectConnectionStrings --theme=LIGHT): ', (answer) => {
                    rl.close();
                    const args = answer.split(' ').map((arg) => {
                        return arg.trim();
                    });
                    void this.stopElectron().then(() => {
                        this.startElectron(args);
                    });
                });
                return;
            }
        });
    }
}
exports.WebpackPluginStartElectron = WebpackPluginStartElectron;
//# sourceMappingURL=webpack-plugin-start-electron.js.map